unit Worker;

interface

uses Classes, Abstraction;

type
  TWorker = class(TThread)
  protected
    FFarmer:TAFarmer;
    FWasteLevel:integer;
  public
    constructor Create(AFarmer:TAFarmer; AWasteLevel:integer); reintroduce;

    procedure Execute; override;
  end;

implementation

uses SysUtils, Strings, Waste;

constructor TWorker.Create(AFarmer:TAFarmer; AWasteLevel:integer);
begin
  FFarmer:=AFarmer;
  FWasteLevel:=AWasteLevel;
  inherited Create(False);
  FreeOnTerminate:=true;  //so we don't have to bother with Free
end;

procedure TWorker.Execute;
var cont:boolean;
    value, limit:PValue;
    sum:TValue;

    tmp:Double;

begin
  FFarmer.AddWorkerRef;  //Register the worker - AddRef

  try
    repeat
      cont:=FFarmer.AskForValues(value, limit);
      if cont then
        begin
          //Yes, we've got some values to sum.
          sum:=0;

          //Sum the values
          while integer(value)<=integer(limit) do
            begin
              sum:=sum+value^;
              inc(value);
              
              //And waste the CPU a little-bit, so we get ran on all processors,
              //if there's enough of threads spawned.
              WasteCPU(random(FWasteLevel));
            end;

          //Summed all up => report the intermediate result 
          FFarmer.ReportSum(sum);

          //And log the progress.
          tmp:=sum; //just to fool the Format routine
          FFarmer.LogMessage(Format(rsWorkerProgress, [ThreadID, tmp]));
        end;

    until Terminated or (not cont);
      //as long as we have to compute      

  finally
    FFarmer.ReleaseWorker;   //Unregister the worker - Release
  end;
end;

end.
